;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Routines for opening and closing doors
;
;------------------------------------------------------------------------------
	include "graphics\graphics_doors.asm"

	DOOR_HEIGHT equ 16
	DOOR_WIDTH equ 16
			
	NUM_DOORS equ 8

	; doors are numbered right to left, bottom to top, to increase the 
	; probability of short circuiting when looking for a collision between 
	; a changing door and the protagonist
	;
	; door 0 - (168, 168)
	; door 1 - (96, 168)
	; door 2 - (64, 168)
	; door 3 - (168, 136)
	; door 4 - (64, 136)
	; door 5 - (128, 112)
	; door 6 - (96, 112)
	; door 7 - (128, 88)
	DOOR_0_X equ 168
	DOOR_0_Y equ 168
	
	DOOR_1_X equ 96
	DOOR_1_Y equ 168
	
	DOOR_2_X equ 64
	DOOR_2_Y equ 168
	
	DOOR_3_X equ 168
	DOOR_3_Y equ 136
	
	DOOR_4_X equ 64
	DOOR_4_Y equ 136
	
	DOOR_5_X equ 128
	DOOR_5_Y equ 112
	
	DOOR_6_X equ 96
	DOOR_6_Y equ 112
	
	DOOR_7_X equ 128
	DOOR_7_Y equ 88
	
	; x values for all doors
	doorXs db DOOR_0_X
		   db DOOR_1_X
		   db DOOR_2_X
		   db DOOR_3_X
		   db DOOR_4_X
		   db DOOR_5_X
		   db DOOR_6_X
		   db DOOR_7_X
	; y values for all doors	   
	doorYs db DOOR_0_Y
		   db DOOR_1_Y
		   db DOOR_2_Y
		   db DOOR_3_Y
		   db DOOR_4_Y
		   db DOOR_5_Y
		   db DOOR_6_Y
		   db DOOR_7_Y
	
	; statuses of each door
	doorIsOpenMany ds 8, 1
	
	; the frame counter value which causes the door to toggle
	currentDoorToggleFrameMany db 250
							   db 80
							   db 226
							   db 170
							   db 10
							   db 120
							   db 200
							   db 50

	; offset from the left-most side of the door, used to check collision
	DOOR_COLLISION_BOX_X_OFFSET equ DOOR_WIDTH/3
	; width of the door area which counts towards collisions
	DOOR_COLLISION_BOX_WIDTH equ DOOR_WIDTH/3
	
	DOOR_0_COLLISION_BOX_X equ DOOR_0_X + DOOR_COLLISION_BOX_X_OFFSET
	DOOR_1_COLLISION_BOX_X equ DOOR_1_X + DOOR_COLLISION_BOX_X_OFFSET
	DOOR_2_COLLISION_BOX_X equ DOOR_2_X + DOOR_COLLISION_BOX_X_OFFSET
	DOOR_3_COLLISION_BOX_X equ DOOR_3_X + DOOR_COLLISION_BOX_X_OFFSET
	DOOR_4_COLLISION_BOX_X equ DOOR_4_X + DOOR_COLLISION_BOX_X_OFFSET
	DOOR_5_COLLISION_BOX_X equ DOOR_5_X + DOOR_COLLISION_BOX_X_OFFSET
	DOOR_6_COLLISION_BOX_X equ DOOR_6_X + DOOR_COLLISION_BOX_X_OFFSET
	DOOR_7_COLLISION_BOX_X equ DOOR_7_X + DOOR_COLLISION_BOX_X_OFFSET
	
	doorCollisionXs db DOOR_0_COLLISION_BOX_X
					db DOOR_1_COLLISION_BOX_X
					db DOOR_2_COLLISION_BOX_X
					db DOOR_3_COLLISION_BOX_X
					db DOOR_4_COLLISION_BOX_X
					db DOOR_5_COLLISION_BOX_X
					db DOOR_6_COLLISION_BOX_X
					db DOOR_7_COLLISION_BOX_X
					
	; Y values of where the protagonist is sent when going through each door
	manDestinationYMany db DOOR_3_Y		; door 0 goes to door 3
						db DOOR_6_Y		; door 1 goes to door 6
						db DOOR_4_Y		; door 2 goes to door 4
						db DOOR_0_Y		; door 3 goes to door 0
						db DOOR_2_Y		; door 4 goes to door 2
						db DOOR_7_Y		; door 5 goes to door 7
						db DOOR_1_Y		; door 6 goes to door 1
						db DOOR_5_Y		; door 7 goes to door 5

						
initialize_doors:
	; draw all closed doors
	ld c, DOOR_0_Y
	ld a, DOOR_0_X
	ld ixl, a
	call draw_closed_door_to_background
	ld c, DOOR_1_Y
	ld a, DOOR_1_X
	ld ixl, a
	call draw_closed_door_to_background
	ld c, DOOR_2_Y
	ld a, DOOR_2_X
	ld ixl, a
	call draw_closed_door_to_background
	ld c, DOOR_3_Y
	ld a, DOOR_3_X
	ld ixl, a
	call draw_closed_door_to_background
	ld c, DOOR_4_Y
	ld a, DOOR_4_X
	ld ixl, a
	call draw_closed_door_to_background
	ld c, DOOR_5_Y
	ld a, DOOR_5_X
	ld ixl, a
	call draw_closed_door_to_background
	ld c, DOOR_6_Y
	ld a, DOOR_6_X
	ld ixl, a
	call draw_closed_door_to_background
	ld c, DOOR_7_Y
	ld a, DOOR_7_X
	ld ixl, a
	call draw_closed_door_to_background
	
	ld d, 0
	ld b, NUM_DOORS
initialize_doors_loop:
	ld e, b
	dec e				; DE := B - 1 (because loop is 1-based)
	
	; Now load currentDoorIsOpen
	; DE = B - 1 at this point, from above
	ld hl, doorIsOpenMany
	add hl, de			; offset by B
	ld (hl), 0			; currentDoorIsOpen := 0
	
	djnz initialize_doors_loop
	ret


; Checks all doors and toggles those which need to be toggled.
;
update_doors:
	ld b, NUM_DOORS
update_doors_loop:
	push bc
	
	ld d, 0
	ld e, b
	dec e				; DE := B - 1 (because loop is 1-based)
	
	; Now load currentDoorX
	; DE = B - 1 at this point, from above
	ld hl, doorXs
	add hl, de			; offset by B
	ld a, (hl)			; A := doorXs[B]
	ld hl, currentDoorX
	ld (hl), a			; currentDoorX := doorXs[B]
	
	; Now load currentDoorY
	; DE = B - 1 at this point, from above
	ld hl, doorYs
	add hl, de			; offset by B
	ld a, (hl)			; A := doorYs[B]
	ld hl, currentDoorY
	ld (hl), a			; currentDoorY := doorYs[B]
	
	; Now load currentDoorIsOpen
	; DE = B - 1 at this point, from above
	ld hl, doorIsOpenMany
	add hl, de			; offset by B
	ld a, (hl)			; A := doorIsOpenMany[B]
	ld hl, currentDoorIsOpen
	ld (hl), a			; currentDoorIsOpen := doorIsOpenMany[B]
	
	; Now load currentDoorToggleFrame
	; DE = B - 1 at this point, from above
	ld hl, currentDoorToggleFrameMany
	add hl, de			; offset by B
	ld a, (hl)			; A := currentDoorToggleFrameMany[B]
	ld hl, currentDoorToggleFrame
	ld (hl), a			; currentDoorToggleFrame := 
						;		currentDoorToggleFrameMany[B]
	
	push de
	call update_door	; now we can update our current door
	pop de
	
	; save potentially modified open/closed value into our door
	ld hl, currentDoorIsOpen
	ld a, (hl)					; A := currentDoorIsOpen
	ld hl, doorIsOpenMany
	add hl, de					; offset by B
	ld (hl), a					; doorIsOpenMany[B] = currentDoorIsOpen
	
	pop bc
	djnz update_doors_loop		; next door
	ret
	

	
	; used as arguments to update_door to update a single door
	currentDoorX db 0
	currentDoorY db 0
	currentDoorIsOpen db 0
	currentDoorToggleFrame db 0

; Update a single door
;
; Input:	
; 		HL - pointer to door's toggle frame value
update_door:
	ld a, (animationFrameCounter)
	cp (hl)								; if current animation frame equals 
	jp z, update_door_perform			; the frame during which we must 
										; toggle this door, then we must toggle
	ret									; not toggling door this frame
update_door_perform:
	ld a, (currentDoorIsOpen)
	cp 0
	jp nz, update_door_close
update_door_open:
	ld a, (currentDoorY)
	ld c, a
	ld a, (currentDoorX)
	ld ixl, a
	call draw_open_door_to_background	; update background buffer
	
	ld a, (currentDoorY)
	ld c, a
	ld a, (currentDoorX)
	ld ixl, a
	ld a, DOOR_HEIGHT
	call copy_buffer_rectangle_to_vram
	ld a, (currentDoorY)
	ld c, a
	ld a, (currentDoorX)
	add a, DOOR_WIDTH/2
	ld ixl, a
	ld a, DOOR_HEIGHT
	call copy_buffer_rectangle_to_vram	; copy background to vram
	
	ld hl, currentDoorIsOpen
	ld (hl), 1							; set door status to open
	
	jp update_door_redraw_man
	
update_door_close:
	ld a, (currentDoorY)
	ld c, a
	ld a, (currentDoorX)
	ld ixl, a
	call draw_closed_door_to_background	; update background buffer
	
	ld a, (currentDoorY)
	ld c, a
	ld a, (currentDoorX)
	ld ixl, a
	ld a, DOOR_HEIGHT
	call copy_buffer_rectangle_to_vram
	ld a, (currentDoorY)
	ld c, a
	ld a, (currentDoorX)
	add a, DOOR_WIDTH/2
	ld ixl, a
	ld a, DOOR_HEIGHT
	call copy_buffer_rectangle_to_vram	; copy background to vram
	
	ld hl, currentDoorIsOpen
	ld (hl), 0							; set door status to closed
	
update_door_redraw_man:
	call schedule_man_display
	ret


; Draws a closed door to the background buffer
;
; Input:	
; 		C - screen Y coordinate
;	  IXL - screen X coordinate
draw_closed_door_to_background:
	push ix
	push bc
	
	ld hl, bitmapClosedDoorLeft
	ld a, DOOR_HEIGHT
	ld de, bitmapDoorLeftMask
	call draw_bitmap_OR_blitted_masked_background
	
	pop bc
	pop ix
	ld de, DOOR_WIDTH/2
	add ix, de					; next vertical slice
	
	ld hl, bitmapClosedDoorRight
	ld a, DOOR_HEIGHT
	ld de, bitmapDoorRightMask
	call draw_bitmap_OR_blitted_masked_background
	
	ret


; Draws an open door to the background buffer
;
; Input:	
; 		C - screen Y coordinate
;	  IXL - screen X coordinate
draw_open_door_to_background:
	push ix
	push bc
	
	ld hl, bitmapOpenDoorLeft
	ld a, DOOR_HEIGHT
	ld de, bitmapDoorLeftMask
	call draw_bitmap_OR_blitted_masked_background
	
	pop bc
	pop ix
	ld de, DOOR_WIDTH/2
	add ix, de					; next vertical slice
	
	ld hl, bitmapOpenDoorRight
	ld a, DOOR_HEIGHT
	ld de, bitmapDoorRightMask
	call draw_bitmap_OR_blitted_masked_background
	
	ret


; Called when the player has pressed the action button, it checks all open 
; doors, and relocates protagonist if it's colliding with an open door
handle_go_through_door:
	ld b, NUM_DOORS
handle_go_through_door_loop:
	push bc
	
	ld d, 0
	ld e, b
	dec e				; DE := B - 1 (because loop is 1-based)
	
	; check if current door is open
	; DE = B - 1 at this point, from above
	ld hl, doorIsOpenMany
	add hl, de			; offset by B
	ld a, (hl)			; A := doorIsOpenMany[B]
	cp 0
	jp z, handle_go_through_door_loop_next_door	; if current door is closed
												; then try next door
	
	; Now load currentDoorX
	; note that door collisionX values are a bit to the right of door X values
	; so that the protagonist needs to overlap the door by a great deal when
	; going through it.
	;
	; DE = B - 1 at this point, from above
	ld hl, doorCollisionXs
	add hl, de			; offset by B
	ld a, (hl)			; A := doorCollisionXs[B]
	ld hl, currentDoorX
	ld (hl), a			; currentDoorX := doorCollisionXs[B]
	
	; Now load currentDoorY
	; DE = B - 1 at this point, from above
	ld hl, doorYs
	add hl, de			; offset by B
	ld a, (hl)			; A := doorYs[B]
	ld hl, currentDoorY
	ld (hl), a			; currentDoorY := doorYs[B]

	push de
	; check collision with protagonist
	ld a, (currentDoorX)
	ld b, a					; B := door collision left
	add a, DOOR_COLLISION_BOX_WIDTH
	ld d, a					; D := door collision right
	ld a, (currentDoorY)
	ld c, a					; C := door top
	add a, DOOR_HEIGHT
	ld e, a					; E := door bottom
	call rectangle_overlap_with_man_test		; A := 1 if man collides with door
	pop de
	cp 0
	jp z, handle_go_through_door_loop_next_door	; if A = 0, we check next door
	
	; there was a collision, so now the protagonist must go through this door

	; look up protagonist's destination Y
	; DE = B - 1 at this point, from above
	ld hl, manDestinationYMany
	add hl, de			; offset by B
	ld a, (hl)			; A := manDestinationYMany[B]
	ld c, a				; C := manDestinationYMany[B]
	
	ld hl, doorXs
	add hl, de			; offset by B
	ld a, (hl)			; A := doorXs[B]
	
	ld d, a
	ld e, c				; setup DE for the "move man" call
						; (align with destination door horizontally)
	call move_man
	call play_go_through_door_sound
	
	pop bc				; remove the extra BC from the stack
						; since we're short-circuiting the loop
	
	ret					; no point in checking any other doors after
						; we've gone through one of them
	
handle_go_through_door_loop_next_door:
	pop bc
	djnz handle_go_through_door_loop	; next door
	; we're out of doors to check
	ret
	

play_go_through_door_sound:
	ld de, 1
	ld bc, 230
	ld a, 3
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 130
	ld a, 3
	call sound_manager_effects_enqueue
	ret