;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Routines used to initialize a new game, when the player presses START
;
;------------------------------------------------------------------------------

	background_screen_definition:
		include "graphics\graphics_background.asm"

	textHolding db 'HELD', 0
	textWin db 'WIN', 0
	textLose db 'LOSE', 0

initialize_new_game:
	call copy_screen_to_background_buffer
	
	call initialize_doors
	call initialize_man
	call initialize_inventory

	call copy_background_buffer_to_video_ram
	call display_gauges
	call initialize_events
	call initialize_snow
	call initialize_trash
	call initialize_score
	
	ld a, COLOUR_BLACK
	call ROM_ROUTINE_SET_BORDER_COLOUR
	
	call interrupt_activate		; turn on the interrupt handler, which is going
								; to play sounds and music during game play
	
	; we're done initializing a new game
	; game play will now begin
	ret
	

copy_screen_to_background_buffer:
	; load the background image into the background buffer
	ld hl, background_screen_definition
	ld de, backgroundVideoBuffer
	call copy_buffer_to_buffer
	ret

	
display_gauges:
	; print HELD next to progress bar
	ld hl, textHolding
	ld b, 7
	ld c, 25
	call text_print_at
	ld b, 7
	ld c, 25
	ld d, FLASH_OFF | BRIGHT_ON | PAPER_BLACK | INK_WHITE
	ld e, 4
	call bitmaps_colours_set_many
	
	; print WIN next to progress bar
	ld hl, textWin
	ld b, 9
	ld c, 26
	call text_print_at
	ld b, 9
	ld c, 26
	ld d, FLASH_OFF | BRIGHT_ON | PAPER_BLACK | INK_WHITE
	ld e, 4
	call bitmaps_colours_set_many
	
	; print LOSE
	ld hl, textLose
	ld b, 14
	ld c, 25
	call text_print_at
	ld b, 14
	ld c, 25
	ld d, FLASH_OFF | BRIGHT_ON | PAPER_BLACK | INK_WHITE
	ld e, 4
	call bitmaps_colours_set_many
	
	ret
	
