;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Routines used for information screens, such as main menu, game lost, and
; game won.
;
;------------------------------------------------------------------------------
	
	FRAMES_TO_WAIT_AFTER_KEYPRESS equ 25
	
	keypressHappened db 0
	framesAfterKeypressCounter db FRAMES_TO_WAIT_AFTER_KEYPRESS
	
; Displays a multi-colour border effect while waiting for the user
; to press the action key.
;
; The various delay values used here, and the seemingly "do nothing"
; operations are all chosen to provide precise timing for the border colours
; to look good.
; 
wait_keypress:
	call interrupt_deactivate		; the sound-playing interrupt interferes
									; with our multi-colour border effect here
									; so we disable it, and play sounds
									; manually after the multi-colour border
									; has been drawn
	ld hl, keypressHappened
	ld (hl), 0
	
wait_keypress_main_loop:
	halt			; wait for an interrupt
	ld a, 0			; black
	call ROM_ROUTINE_SET_BORDER_COLOUR
	
	; delay to draw the initial black region at the top of the screen
	ld b, 187
wait_keypress_delay_black_strip_top:
	add hl, hl
	add hl, hl
	add hl, hl
	add hl, hl
	add hl, hl
	add hl, hl
	add a, b
	djnz wait_keypress_delay_black_strip_top
	
	ld a, 2			; red
	call ROM_ROUTINE_SET_BORDER_COLOUR
	
	; delay to draw the red strip
	ld b, 20
wait_keypress_delay_red_strip:
	add hl, hl
	add hl, hl
	add hl, hl
	add a, b
	djnz wait_keypress_delay_red_strip
	
	ld a, 6			; yellow
	call ROM_ROUTINE_SET_BORDER_COLOUR
	
	; delay to draw the yellow strip
	ld b, 22
wait_keypress_delay_yellow_strip:
	add hl, hl
	add hl, hl
	add hl, hl
	add a, b
	djnz wait_keypress_delay_yellow_strip
	
	ld a, 4			; green
	call ROM_ROUTINE_SET_BORDER_COLOUR
	
	; delay to draw the green strip
	ld b, 21
wait_keypress_delay_green_strip:
	add hl, hl
	add hl, hl
	add hl, hl
	add a, b
	djnz wait_keypress_delay_green_strip
	
	ld a, 5			; blue
	call ROM_ROUTINE_SET_BORDER_COLOUR
	
	; delay to draw the blue strip
	ld b, 20
wait_keypress_delay_blue_strip:
	add hl, hl
	add hl, hl
	add hl, hl
	add a, b
	djnz wait_keypress_delay_blue_strip
	
	ld a, 0						; border is black below the coloured strips
	call ROM_ROUTINE_SET_BORDER_COLOUR
	
	; done drawing the multi-coloured border
	; now perform other tasks
	
	; rotate colours of the title by setting the ink to values between
	; 0 and 7 inclusive
	ld a, (menuFrameCounter)
	srl a
	srl a
	and %00000111		; A := (menuFrameCounter/4) mod 8
	or FLASH_OFF | PAPER_BLACK	; "add in" the other attributes
	ld d, a

	ld a, (menuFrameCounter)
	srl a
	and %00000001		; every two frames change brightness
	xor %00000001		; flip value to cycle dark->bright
	sla a
	sla a				; in essence, we're flipping through this sequence
	sla a				; blue -> bright blue -> red -> bright red -> etc.
	sla a				; by switching the colour every 4 frames
	sla a				; and the brightness from 0 to 1 every 2 frames
	sla a
	or d
	
	ld d, a				; D := attribute value
	ld b, 2
	ld c, 9
	ld e, 14
	call bitmaps_colours_set_many
	
	; advance random number sequence
	call get_next_random
	
	; increment counter
	ld hl, menuFrameCounter
	inc (hl)
	
	; play sounsd and music manually (since interrupt handler is disabled)
	call sound_manager_continue_playing
	
	ld a, (keypressHappened)
	cp 1
	jp z, wait_keypress_counting_down	; if key was pressed before, count down
	
	; see if the "start" key was pressed
	call keyboard_read_keys
	ld b, KEY_M
	call check_just_pressed_keys_b_n_m_ss_sp
	cp 0
	jp z, wait_keypress_start_key_not_pressed
	
	
	; start key was pressed
	call play_keypress_sound
	ld hl, framesAfterKeypressCounter
	ld (hl), FRAMES_TO_WAIT_AFTER_KEYPRESS		; start countdown
	ld hl, keypressHappened						
	ld (hl), 1									; set flag
	
wait_keypress_counting_down:
	ld a, (framesAfterKeypressCounter)
	dec a
	ld (framesAfterKeypressCounter), a	; countdown--
	cp 0
	jp nz, wait_keypress_main_loop	; if countdown is still going
									; then go to beginning
	
	; key was pressed before, and countdown has finished
	; we're done altogether
	ret
	
wait_keypress_start_key_not_pressed:
	jp wait_keypress_main_loop

	
play_keypress_sound:
	ld de, 5
	ld bc, 330
	ld a, 10
	call sound_manager_effects_enqueue
	ret
	
