;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Functionality for picking up items and displaying currently held item
;
;------------------------------------------------------------------------------

	; we want to erase the previous item, so the mask must cover 
	; the entire square
	bitmapItemMask
			db %11111111
			db %11111111
			db %11111111
			db %11111111
			db %11111111
			db %11111111
			db %11111111
			db %11111111
	bitmapEmpty ds 8, 0		; 8 empty lines
	
	bitmapPlunger
			db %00000001
			db %00000010
			db %00000100
			db %01111000
			db %11111000
			db %01111000
			db %00111000
			db %00010000
	bitmapShovel
			db %00000100
			db %00000010
			db %00000101
			db %01101000
			db %11110000
			db %11111000
			db %01111000
			db %00110000
	bitmapGarbage
			db %00101100
			db %00011000
			db %00111100
			db %01010110
			db %10101011
			db %11010101
			db %10101011
			db %01111110

	ITEM_GAUGE_X equ 240
	ITEM_GAUGE_Y equ 56
	
	PLUNGER_LOCATION_LEFT equ 162
	PLUNGER_LOCATION_RIGHT equ 165
	PLUNGER_LOCATION_TOP equ 96
	PLUNGER_LOCATION_BOTTOM equ 104
	
	SHOVEL_LOCATION_LEFT equ PLUNGER_LOCATION_LEFT + 3*8 ; 3 characters away
	SHOVEL_LOCATION_RIGHT equ PLUNGER_LOCATION_RIGHT + 3*8
	SHOVEL_LOCATION_TOP equ PLUNGER_LOCATION_TOP
	SHOVEL_LOCATION_BOTTOM equ PLUNGER_LOCATION_BOTTOM
	
	DUMPSTER_X equ 240
	DUMPSTER_Y equ 168
	
	ITEM_NONE equ 0
	ITEM_PLUNGER equ 1
	ITEM_SHOVEL equ 2
	ITEM_GARBAGE equ 3
	
	itemHeld db 255

	
initialize_inventory:
	call pickup_clear
	ret


pickup_shovel:
	ld a, (itemHeld)
	cp ITEM_GARBAGE
	jp z, pickup_shovel_done	; if holding garbage, we cannot pick up shovel
	
	call play_pickup_sound
	ld hl, itemHeld
	ld (hl), ITEM_SHOVEL
	call inventory_display_shovel
pickup_shovel_done:
	ret

	
pickup_plunger:
	ld a, (itemHeld)
	cp ITEM_GARBAGE
	jp z, pickup_plunger_done	; if holding garbage, we cannot pick up plunger
	
	call play_pickup_sound
	ld hl, itemHeld
	ld (hl), ITEM_PLUNGER
	call inventory_display_plunger
pickup_plunger_done:
	ret
	
	
pickup_garbage:
	ld hl, itemHeld
	ld (hl), ITEM_GARBAGE
	call inventory_display_garbage
	ret

	
pickup_clear:
	ld hl, itemHeld
	ld (hl), ITEM_NONE
	call inventory_display_clear
	ret
	
; Called when the player has pressed the action button, it checks 
; whether the player collides with the plunger.
handle_pickup_plunger:
	ld b, PLUNGER_LOCATION_LEFT
	ld d, PLUNGER_LOCATION_RIGHT
	ld c, PLUNGER_LOCATION_TOP
	ld e, PLUNGER_LOCATION_BOTTOM
	call rectangle_overlap_with_man_test	; A := 1 if there was a collision
	cp 0
	jp z, handle_pickup_plunger_no_collision
	
	call pickup_plunger
handle_pickup_plunger_no_collision:
	ret
	
	
; Called when the player has pressed the action button, it checks 
; whether the player collides with the shovel.
handle_pickup_shovel:
	ld b, SHOVEL_LOCATION_LEFT
	ld d, SHOVEL_LOCATION_RIGHT
	ld c, SHOVEL_LOCATION_TOP
	ld e, SHOVEL_LOCATION_BOTTOM
	call rectangle_overlap_with_man_test	; A := 1 if there was a collision
	cp 0
	jp z, handle_pickup_shovel_no_collision
	
	call pickup_shovel
handle_pickup_shovel_no_collision:
	ret
	
	
inventory_display_plunger:
	ld hl, bitmapPlunger
	ld de, bitmapItemMask
	ld ixl, ITEM_GAUGE_X
	ld c, ITEM_GAUGE_Y
	ld a, 8
	call draw_bitmap_OR_blitted_masked_vram
	ret

inventory_display_garbage:
	ld hl, bitmapGarbage
	ld de, bitmapItemMask
	ld ixl, ITEM_GAUGE_X
	ld c, ITEM_GAUGE_Y
	ld a, 8
	call draw_bitmap_OR_blitted_masked_vram
	ret
	
inventory_display_shovel:
	ld hl, bitmapShovel
	ld de, bitmapItemMask
	ld ixl, ITEM_GAUGE_X
	ld c, ITEM_GAUGE_Y
	ld a, 8
	call draw_bitmap_OR_blitted_masked_vram
	ret

inventory_display_clear:
	ld hl, bitmapEmpty
	ld de, bitmapItemMask
	ld ixl, ITEM_GAUGE_X
	ld c, ITEM_GAUGE_Y
	ld a, 8
	call draw_bitmap_OR_blitted_masked_vram
	ret
	

; Called when the player has pressed the action button, it checks 
; whether the player collides with the garbage dumpster and then attempts 
; to handle the dumping.
;
; Output:
; 		A - 1 if protagonist is colliding with the dumpster, 0 otherwise
handle_dump_garbage:
	ld a, (manX)
	cp DUMPSTER_X-16	; man has to be directly to the left of the dumpster
	jp nz, handle_dump_garbage_not_handled
	
	ld a, (manY)
	cp DUMPSTER_Y		; man's top Y equals dumpster's top Y
	jp nz, handle_dump_garbage_not_handled

	ld a, (itemHeld)
	cp ITEM_GARBAGE
	jp nz, handle_dump_garbage_not_handled

	call pickup_clear	; dump the held garbage
	call play_trash_into_dumpster_sound

	ld hl, hasDroppedTrashAlready
	ld (hl), 0						; reset flag
	
	ld a, 1
	ret
handle_dump_garbage_not_handled:
	ld a, 0
	ret


play_trash_into_dumpster_sound:
	ld de, 1
	ld bc, 150
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 4
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 150
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 200
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 4
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 200
	ld a, 7
	call sound_manager_effects_enqueue
	ret
	

play_pickup_sound:
	ld de, 1
	ld bc, 100
	ld a, 3
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 3
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 180
	ld a, 3
	call sound_manager_effects_enqueue
	ret
	
