;------------------------------------------------------------------------------
;
;        Husband Chores
;                      a ZX Spectrum game by Sebastian Mihai, 2016
;                                        http://sebastianmihai.com
;
; As a kid, I was fascinated by my ZX Spectrum, and I used BASIC to create 
; small video games. However, I knew that almost all professional games were
; made using Z80 assembly language, which I tried, unsuccessfully, to learn
; myself.
; 
; Husband Chores represents my return to the computer which not only introduced
; me to the world of video games, but also kick-started my software development
; career.
;
; In Husband Chores, the player must keep the house clean using various tools.
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; This is the outer-most file of Husband Chores, which defines the entry point
; into the program.
;
; It is responsible with the inclusion of all other source files, setting up 
; a stack, other fundamental initialization, and running the following:
;
;  main.asm -->-+
;(this file)    |
;               v
;        +-->info_screen_menu_main.asm
;        |      |                     
;        |      |               +---> info_screen_game_won.asm -->--+
;        |      v               |                                   |
;        |  game_main.asm-->----+                                   v
;        |                      |                                   |
;        |                      +---> info_screen_game_lost.asm ->--+
;        |                                                          |
;        +--<------------<--------------<--------------<------------+
; 
;------------------------------------------------------------------------------

main: org 33000			; stay above ULA-contended memory

	jp start
	
	author db "Husband Chores is written by Sebastian Mihai, 2016"
	
	include "libzx\libzx.asm"			; include the libzx library
	
	include "music.asm"
	include "protagonist.asm"
	include "score.asm"
	include "doors.asm"
	include "inventory.asm"
	include "obstructions.asm"
	include "events.asm"
	include "snow.asm"
	include "trash.asm"
	
	include "info_screen.asm"
	include "info_screen_menu.asm"
	include "info_screen_game_won.asm"
	include "info_screen_game_lost.asm"
	
	include "game_initialization.asm"
	include "game_main.asm"

	stack ds 512		; reserve some bytes for our stack
	endstack:			; mark end of our stack
	
start:
	ld sp, endstack				; set up our new stack (we point to the end 
								; because the stack grows into lower addresses)
	
	call initialize_libzx		; initialize libzx library
	call initialize_game_music
	; the interrupt handler is in charge of playing sounds during the game
	call initialize_interrupt_handler
	
;------------------------------------------------------------------------------
; This is the outer-most loop in the program
;------------------------------------------------------------------------------	
outer_main_loop:
	call menu_begin
	call game_begin
	jp outer_main_loop

	
	include "interrupt.asm"		; interrupt routines and data are ORG'd
								; in the high areas of memory, which is why
								; they're included after all other game code

end main	; this is needed so the Pasmo assembler generates an
			; "auto-running" BASIC header for us
			;
			; the only way on the ZX Spectrum to run a program automatically
			; after loading it (that is, without requiring the user to issue
			; a "RUN" command manually), is via a loader written in BASIC
			; which simply loads the main program block (written in assembler)
			; after which it jumps into its beginning
