;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Functionality for restricting the movement of the protagonist via various
; collision checks.
;
;------------------------------------------------------------------------------

	LEFT_WALL_LEFT equ 56
	LEFT_WALL_TOP equ 88
	LEFT_WALL_RIGHT equ 63
	LEFT_WALL_BOTTOM equ 183
	
	RIGHT_WALL_LEFT equ 192
	RIGHT_WALL_TOP equ 88
	RIGHT_WALL_RIGHT equ 199
	RIGHT_WALL_BOTTOM equ 151
	
	DUMPSTER_LEFT equ 240
	DUMPSTER_TOP equ 168
	DUMPSTER_RIGHT equ 247
	DUMPSTER_BOTTOM equ 183
	
	SECOND_FLOOR_LEFT equ 112
	SECOND_FLOOR_TOP equ 144
	SECOND_FLOOR_RIGHT equ 143
	SECOND_FLOOR_BOTTOM equ 151
	
	THIRD_FLOOR_1_LEFT equ 64
	THIRD_FLOOR_1_TOP equ 120
	THIRD_FLOOR_1_RIGHT equ 71
	THIRD_FLOOR_1_BOTTOM equ 127
	
	THIRD_FLOOR_2_LEFT equ 184
	THIRD_FLOOR_2_TOP equ 120
	THIRD_FLOOR_2_RIGHT equ 191
	THIRD_FLOOR_2_BOTTOM equ 127
	
	FOURTH_FLOOR_LEFT equ 96
	FOURTH_FLOOR_TOP equ 96
	FOURTH_FLOOR_RIGHT equ 103
	FOURTH_FLOOR_BOTTOM equ 103
	
; Meant to be called after manX or manY change
;
; Output:
; 		A = 1 when the man's current location is invalid (inside a wall, etc.)
;       A = 0 otherwise
is_man_obstructed:
	call check_dumpster_obstruction
	cp 0
	jp nz, is_man_obstructed_true
	
	call check_second_floor_obstruction
	cp 0
	jp nz, is_man_obstructed_true

	call check_third_floor_1_obstruction
	cp 0
	jp nz, is_man_obstructed_true
	
	call check_third_floor_2_obstruction
	cp 0
	jp nz, is_man_obstructed_true
	
	call check_fourth_floor_obstruction
	cp 0
	jp nz, is_man_obstructed_true
	
	call check_left_wall_obstruction
	cp 0
	jp nz, is_man_obstructed_true
	
	call check_right_wall_obstruction
	cp 0
	jp nz, is_man_obstructed_true
	
	call check_snow_pile_obstruction
	cp 0
	jp nz, is_man_obstructed_true
	
	; no obstruction
	ld a, 0
	ret
is_man_obstructed_true:
	ld a, 1
	ret


; Third floor, right most
; Output:
; 		A = 1 when obstructed	
check_third_floor_2_obstruction:
	ld b, THIRD_FLOOR_2_LEFT
	ld c, THIRD_FLOOR_2_TOP
	ld d, THIRD_FLOOR_2_RIGHT
	ld e, THIRD_FLOOR_2_BOTTOM
	call rectangle_overlap_with_man_test
	ret
	

; Third floor, left most	
; Output:
; 		A = 1 when obstructed	
check_third_floor_1_obstruction:
	ld b, THIRD_FLOOR_1_LEFT
	ld c, THIRD_FLOOR_1_TOP
	ld d, THIRD_FLOOR_1_RIGHT
	ld e, THIRD_FLOOR_1_BOTTOM
	call rectangle_overlap_with_man_test
	ret
	

; Second floor, both objects
; Output:
; 		A = 1 when obstructed	
check_second_floor_obstruction:
	ld b, SECOND_FLOOR_LEFT
	ld c, SECOND_FLOOR_TOP
	ld d, SECOND_FLOOR_RIGHT
	ld e, SECOND_FLOOR_BOTTOM
	call rectangle_overlap_with_man_test
	ret


; Output:
; 		A = 1 when obstructed	
check_fourth_floor_obstruction:
	ld b, FOURTH_FLOOR_LEFT
	ld c, FOURTH_FLOOR_TOP
	ld d, FOURTH_FLOOR_RIGHT
	ld e, FOURTH_FLOOR_BOTTOM
	call rectangle_overlap_with_man_test
	ret
	

; Output:
; 		A = 1 when obstructed	
check_left_wall_obstruction:
	ld b, LEFT_WALL_LEFT
	ld c, LEFT_WALL_TOP
	ld d, LEFT_WALL_RIGHT
	ld e, LEFT_WALL_BOTTOM
	call rectangle_overlap_with_man_test
	ret


; Output:
; 		A = 1 when obstructed	
check_right_wall_obstruction:
	ld b, RIGHT_WALL_LEFT
	ld c, RIGHT_WALL_TOP
	ld d, RIGHT_WALL_RIGHT
	ld e, RIGHT_WALL_BOTTOM
	call rectangle_overlap_with_man_test
	ret
	

; Output:
; 		A = 1 when obstructed	
check_dumpster_obstruction:
	ld b, DUMPSTER_LEFT
	ld c, DUMPSTER_TOP
	ld d, DUMPSTER_RIGHT
	ld e, DUMPSTER_BOTTOM
	call rectangle_overlap_with_man_test
	ret
	

; Output:
; 		A = 1 when obstructed	
check_snow_pile_obstruction:
	ld a, (snowPresent)
	cp 0
	jp nz, check_snow_pile_obstruction_perform
	
	ld a, 0			; when snow is not present, we cannot collide with it
	ret
check_snow_pile_obstruction_perform:
	ld b, SNOW_X
	ld c, SNOW_Y
	ld d, SNOW_X+16
	ld e, SNOW_Y+8
	call rectangle_overlap_with_man_test
	ret